@@ -16,22 +16,27 @@
 #ifndef QGSLAYOUTEXPORTER_H
 #define QGSLAYOUTEXPORTER_H
 
-#include <QPrinter>
 #include "qgis_core.h"
 #include "qgsmargins.h"
 #include "qgslayoutrendercontext.h"
 #include "qgslayoutreportcontext.h"
+#include "qgslayoutitem.h"
 #include <QPointer>
 #include <QSize>
 #include <QRectF>
+#include <QVector>
+#include <functional>
 
 #ifndef QT_NO_PRINTER
+#include <QPrinter>
+#endif
 
 class QgsLayout;
 class QPainter;
 class QgsLayoutItemMap;
 class QgsAbstractLayoutIterator;
 class QgsFeedback;
+class QgsLabelingResults;
 
 /**
  * \ingroup core
@@ -65,7 +70,7 @@ class CORE_EXPORT QgsLayoutExporter
      */
     QgsLayoutExporter( QgsLayout *layout );
 
-    virtual ~QgsLayoutExporter() = default;
+    virtual ~QgsLayoutExporter();
 
     /**
      * Returns the layout linked to this exporter.
@@ -206,7 +211,14 @@ class CORE_EXPORT QgsLayoutExporter
       /**
        * Layout context flags, which control how the export will be created.
        */
-      QgsLayoutRenderContext::Flags flags = nullptr;
+      QgsLayoutRenderContext::Flags flags = QgsLayoutRenderContext::Flags();
+
+      /**
+       * A list of predefined scales to use with the layout. This is used
+       * for maps which are set to the predefined atlas scaling mode.
+       * \since QGIS 3.10
+       */
+      QVector<qreal> predefinedMapScales;
 
     };
 
@@ -265,6 +277,13 @@ class CORE_EXPORT QgsLayoutExporter
        */
       bool forceVectorOutput = false;
 
+      /**
+       * Indicates whether PDF export should append georeference data
+       *
+       * \since QGIS 3.10
+       */
+      bool appendGeoreference = true;
+
       /**
        * Indicates whether PDF export should include metadata generated
        * from the layout's project's metadata.
@@ -276,7 +295,7 @@ class CORE_EXPORT QgsLayoutExporter
       /**
        * Layout context flags, which control how the export will be created.
        */
-      QgsLayoutRenderContext::Flags flags = nullptr;
+      QgsLayoutRenderContext::Flags flags = QgsLayoutRenderContext::Flags();
 
       /**
        * Text rendering format, which controls how text should be rendered in the export (e.g.
@@ -284,7 +303,88 @@ class CORE_EXPORT QgsLayoutExporter
        *
        * \since QGIS 3.4.3
        */
-      QgsRenderContext::TextRenderFormat textRenderFormat = QgsRenderContext::TextFormatAlwaysOutlines;
+      Qgis::TextRenderFormat textRenderFormat = Qgis::TextRenderFormat::AlwaysOutlines;
+
+      /**
+       * Indicates whether vector geometries should be simplified to avoid redundant extraneous detail,
+       * such as vertices which are not visible at the specified dpi of the output.
+       *
+       * \since QGIS 3.10
+       */
+      bool simplifyGeometries = true;
+
+      /**
+       * TRUE if GeoPDF files should be created, instead of normal PDF files.
+       *
+       * Whilst GeoPDF files can include some desirable properties like the ability to interactively
+       * query map features, they also can result in lower-quality output files, or forced rasterization
+       * of layers.
+       *
+       * \note Requires builds based on GDAL 3.0 or greater.
+       *
+       * \since QGIS 3.10
+       */
+      bool writeGeoPdf = false;
+
+      /**
+       * TRUE if individual layers from the layout should be rendered to separate PDF files.
+       *
+       * This option allows for separation of logic layout layers to individual PDF files. For instance,
+       * if this option is TRUE, then a separate PDF file will be created per layer per map item in the
+       * layout. Additionally, separate PDF files may be created for other complex layout items, resulting
+       * in a set of PDF files which contain logical atomic components of the layout.
+       *
+       * This option is designed to allow the PDF files to be composited back together in an external
+       * application (e.g. Adobe Illustrator) as a non-QGIS, post-production step.
+       *
+       * \since QGIS 3.14
+       */
+      bool exportLayersAsSeperateFiles = false; // TODO QGIS 4 fix typo  //#spellok
+
+      /**
+       * TRUE if ISO3200 extension format georeferencing should be used.
+       *
+       * This is a recommended setting which results in Geospatial PDF files compatible
+       * with the built-in Acrobat geospatial tools.
+       *
+       * If PdfExportSettings::writeGeoPdf is FALSE than this option has no effect.
+       */
+      bool useIso32000ExtensionFormatGeoreferencing = true;
+
+      /**
+       * TRUE if OGC "best practice" format georeferencing should be used.
+       *
+       * \warning This results in GeoPDF files compatible with the TerraGo suite of tools, but
+       * can break compatibility with the built-in Acrobat geospatial tools (yes, GeoPDF
+       * format is a mess!).
+       *
+       * If PdfExportSettings::writeGeoPdf is FALSE than this option has no effect.
+      */
+      bool useOgcBestPracticeFormatGeoreferencing = false;
+
+      /**
+       * TRUE if feature vector information (such as attributes) should be exported during GeoPDF exports.
+       *
+       * If PdfExportSettings::writeGeoPdf is FALSE than this option has no effect.
+       */
+      bool includeGeoPdfFeatures = true;
+
+      /**
+       * Optional list of map themes to export as GeoPDF layer groups.
+       *
+       * If set, map item's which are not assigned a specific map theme will iterate through all listed
+       * themes and a GeoPDF layer group will be created for each.
+       *
+       * If PdfExportSettings::writeGeoPdf is FALSE than this option has no effect.
+       */
+      QStringList exportThemes;
+
+      /**
+       * A list of predefined scales to use with the layout. This is used
+       * for maps which are set to the predefined atlas scaling mode.
+       * \since QGIS 3.10
+       */
+      QVector<qreal> predefinedMapScales;
 
     };
 
@@ -349,10 +449,19 @@ class CORE_EXPORT QgsLayoutExporter
       /**
        * Layout context flags, which control how the export will be created.
        */
-      QgsLayoutRenderContext::Flags flags = nullptr;
+      QgsLayoutRenderContext::Flags flags = QgsLayoutRenderContext::Flags();
+
+      /**
+       * A list of predefined scales to use with the layout. This is used
+       * for maps which are set to the predefined atlas scaling mode.
+       * \since QGIS 3.10
+       */
+      QVector<qreal> predefinedMapScales;
 
     };
 
+#ifndef QT_NO_PRINTER
+
     /**
      * Prints the layout to a \a printer, using the specified export \a settings.
      *
@@ -371,7 +480,7 @@ class CORE_EXPORT QgsLayoutExporter
     static ExportResult print( QgsAbstractLayoutIterator *iterator, QPrinter &printer,
                                const QgsLayoutExporter::PrintExportSettings &settings,
                                QString &error SIP_OUT, QgsFeedback *feedback = nullptr );
-
+#endif
 
     //! Contains settings relating to exporting layouts to SVG
     struct SvgExportSettings
@@ -412,6 +521,16 @@ class CORE_EXPORT QgsLayoutExporter
        */
       bool exportAsLayers = false;
 
+      /**
+       * Set to TRUE to export labels to separate layers (grouped by map layer)
+       * in layered SVG exports.
+       *
+       * This option is only used if exportAsLayers is TRUE.
+       *
+       * \since QGIS 3.10
+       */
+      bool exportLabelsToSeparateLayers = true;
+
       /**
        * Indicates whether SVG export should include RDF metadata generated
        * from the layout's project's metadata.
@@ -423,7 +542,7 @@ class CORE_EXPORT QgsLayoutExporter
       /**
        * Layout context flags, which control how the export will be created.
        */
-      QgsLayoutRenderContext::Flags flags = nullptr;
+      QgsLayoutRenderContext::Flags flags = QgsLayoutRenderContext::Flags();
 
       /**
        * Text rendering format, which controls how text should be rendered in the export (e.g.
@@ -431,8 +550,22 @@ class CORE_EXPORT QgsLayoutExporter
        *
        * \since QGIS 3.4.3
        */
-      QgsRenderContext::TextRenderFormat textRenderFormat = QgsRenderContext::TextFormatAlwaysOutlines;
+      Qgis::TextRenderFormat textRenderFormat = Qgis::TextRenderFormat::AlwaysOutlines;
 
+      /**
+       * Indicates whether vector geometries should be simplified to avoid redundant extraneous detail,
+       * such as vertices which are not visible at the specified dpi of the output.
+       *
+       * \since QGIS 3.10
+       */
+      bool simplifyGeometries = true;
+
+      /**
+       * A list of predefined scales to use with the layout. This is used
+       * for maps which are set to the predefined atlas scaling mode.
+       * \since QGIS 3.10
+       */
+      QVector<qreal> predefinedMapScales;
     };
 
     /**
@@ -464,6 +597,29 @@ class CORE_EXPORT QgsLayoutExporter
      */
     QString errorFile() const { return mErrorFileName; }
 
+    /**
+     * Returns the labeling results for all map items included in the export. Map keys are the item UUIDs (see QgsLayoutItem::uuid()).
+     *
+     * Ownership of the results remains with the layout exporter.
+     *
+     * \since QGIS 3.20
+     */
+    QMap< QString, QgsLabelingResults * > labelingResults();
+
+#ifndef SIP_RUN
+
+    /**
+     * Takes the labeling results for all map items included in the export. Map keys are the item UUIDs (see QgsLayoutItem::uuid()).
+     *
+     * Ownership of the results is transferred to the caller.
+     *
+     * \note Not available in Python bindings
+     *
+     * \since QGIS 3.20
+     */
+    QMap< QString, QgsLabelingResults * > takeLabelingResults();
+#endif
+
     /**
      * Georeferences a \a file (image of PDF) exported from the layout.
      *
@@ -497,6 +653,25 @@ class CORE_EXPORT QgsLayoutExporter
      */
     void computeWorldFileParameters( const QRectF &region, double &a, double &b, double &c, double &d, double &e, double &f, double dpi = -1 ) const;
 
+    /**
+     * Returns TRUE if the specified \a layout contains visible items which have settings
+     * that require rasterization of the entire export layout in order to reproduce the desired
+     * appearance.
+     *
+     * \see containsAdvancedEffects()
+     * \since QGIS 3.20
+     */
+    static bool requiresRasterization( const QgsLayout *layout );
+
+    /**
+     * Returns TRUE if the specified \a layout contains visible items which have settings
+     * such as opacity which will prevent these individual items from being exported as vector artwork.
+     *
+     * \see requiresRasterization()
+     * \since QGIS 3.20
+     */
+    static bool containsAdvancedEffects( const QgsLayout *layout );
+
   protected:
 
     /**
@@ -510,6 +685,9 @@ class CORE_EXPORT QgsLayoutExporter
 
     QPointer< QgsLayout > mLayout;
 
+    void captureLabelingResults();
+    QMap< QString, QgsLabelingResults * > mLabelingResults;
+
     mutable QString mErrorFileName;
 
     QImage createImage( const ImageExportSettings &settings, int page, QRectF &bounds, bool &skipPage ) const;
@@ -543,6 +721,8 @@ class CORE_EXPORT QgsLayoutExporter
     //! Write a world file
     void writeWorldFile( const QString &fileName, double a, double b, double c, double d, double e, double f ) const;
 
+#ifndef QT_NO_PRINTER
+
     /**
      * Prepare a \a printer for printing a layout as a PDF, to the destination \a filePath.
      */
@@ -566,9 +746,10 @@ class CORE_EXPORT QgsLayoutExporter
     ExportResult printPrivate( QPrinter &printer, QPainter &painter, bool startNewPage = false, double dpi = -1, bool rasterize = false );
 
     static void updatePrinterPageSize( QgsLayout *layout, QPrinter &printer, int page );
+#endif
 
     ExportResult renderToLayeredSvg( const SvgExportSettings &settings, double width, double height, int page, const QRectF &bounds,
-                                     const QString &filename, int svgLayerId, const QString &layerName,
+                                     const QString &filename, unsigned int svgLayerId, const QString &layerName,
                                      QDomDocument &svg, QDomNode &svgDocRoot, bool includeMetadata ) const;
 
     void appendMetadataToSvg( QDomDocument &svg ) const;
@@ -576,12 +757,14 @@ class CORE_EXPORT QgsLayoutExporter
     bool georeferenceOutputPrivate( const QString &file, QgsLayoutItemMap *referenceMap = nullptr,
                                     const QRectF &exportRegion = QRectF(), double dpi = -1, bool includeGeoreference = true, bool includeMetadata = false ) const;
 
+    ExportResult handleLayeredExport( const QList<QGraphicsItem *> &items, const std::function<QgsLayoutExporter::ExportResult( unsigned int layerId, const QgsLayoutItem::ExportLayerDetail &layerDetails )> &exportFunc );
+
+    static QgsVectorSimplifyMethod createExportSimplifyMethod();
     friend class TestQgsLayout;
+    friend class TestQgsLayoutExporter;
 
 };
 
-#endif // ! QT_NO_PRINTER
-
 #endif //QGSLAYOUTEXPORTER_H
 
 
