@@ -31,7 +31,7 @@ class QgsRectangle;
 
 /**
  * \ingroup core
- * Raster data container.
+ * \brief Raster data container.
  */
 class CORE_EXPORT QgsRasterBlock
 {
@@ -53,7 +53,7 @@ class CORE_EXPORT QgsRasterBlock
      *  \param dataType raster data type
      *  \param width width of data matrix
      *  \param height height of data matrix
-     *  \returns true on success
+     *  \returns TRUE on success
      */
     bool reset( Qgis::DataType dataType, int width, int height );
 
@@ -61,51 +61,53 @@ class CORE_EXPORT QgsRasterBlock
     // and works also if block is valid but empty - difference between valid and empty?
 
     /**
-     * \brief Returns true if the block is valid (correctly filled with data).
+     * \brief Returns TRUE if the block is valid (correctly filled with data).
      *  An empty block may still be valid (if zero size block was requested).
      *  If the block is not valid, error may be retrieved by error() method.
      */
-    bool isValid() const { return mValid; }
+    bool isValid() const SIP_HOLDGIL { return mValid; }
 
     //! \brief Mark block as valid or invalid
-    void setValid( bool valid ) { mValid = valid; }
+    void setValid( bool valid ) SIP_HOLDGIL { mValid = valid; }
 
     /**
-     * Returns true if block is empty, i.e. its size is 0 (zero rows or cols).
-     *  This method does not return true if size is not zero and all values are
+     * Returns TRUE if block is empty, i.e. its size is 0 (zero rows or cols).
+     *  This method does not return TRUE if size is not zero and all values are
      *  'no data' (null).
      */
     bool isEmpty() const;
 
-    // Return data type size in bytes
-    static int typeSize( int dataType )
+    /**
+     * Returns the size in bytes for the specified \a dataType.
+     */
+    static int typeSize( Qgis::DataType dataType ) SIP_HOLDGIL
     {
       // Modified and extended copy from GDAL
       switch ( dataType )
       {
-        case Qgis::Byte:
+        case Qgis::DataType::Byte:
           return 1;
 
-        case Qgis::UInt16:
-        case Qgis::Int16:
+        case Qgis::DataType::UInt16:
+        case Qgis::DataType::Int16:
           return 2;
 
-        case Qgis::UInt32:
-        case Qgis::Int32:
-        case Qgis::Float32:
-        case Qgis::CInt16:
+        case Qgis::DataType::UInt32:
+        case Qgis::DataType::Int32:
+        case Qgis::DataType::Float32:
+        case Qgis::DataType::CInt16:
           return 4;
 
-        case Qgis::Float64:
-        case Qgis::CInt32:
-        case Qgis::CFloat32:
+        case Qgis::DataType::Float64:
+        case Qgis::DataType::CInt32:
+        case Qgis::DataType::CFloat32:
           return 8;
 
-        case Qgis::CFloat64:
+        case Qgis::DataType::CFloat64:
           return 16;
 
-        case Qgis::ARGB32:
-        case Qgis::ARGB32_Premultiplied:
+        case Qgis::DataType::ARGB32:
+        case Qgis::DataType::ARGB32_Premultiplied:
           return 4;
 
         default:
@@ -113,160 +115,367 @@ class CORE_EXPORT QgsRasterBlock
       }
     }
 
-    // Data type in bytes
-    int dataTypeSize() const
+    /**
+     * Data type size in bytes.
+     */
+    int dataTypeSize() const SIP_HOLDGIL
     {
       return typeSize( mDataType );
     }
 
-    //! Returns true if data type is numeric
+    //! Returns TRUE if data type is numeric
     static bool typeIsNumeric( Qgis::DataType type );
 
-    //! Returns true if data type is color
+    //! Returns TRUE if data type is color
     static bool typeIsColor( Qgis::DataType type );
 
     //! Returns data type
-    Qgis::DataType dataType() const { return mDataType; }
+    Qgis::DataType dataType() const  SIP_HOLDGIL { return mDataType; }
 
     //! For given data type returns wider type and sets no data value
     static Qgis::DataType typeWithNoDataValue( Qgis::DataType dataType, double *noDataValue );
 
     /**
-     * True if the block has no data value.
-     * \returns true if the block has no data value
+     * TRUE if the block has no data value.
+     * \returns TRUE if the block has no data value
      * \see noDataValue(), setNoDataValue(), resetNoDataValue()
      */
-    bool hasNoDataValue() const { return mHasNoDataValue; }
+    bool hasNoDataValue() const SIP_HOLDGIL { return mHasNoDataValue; }
 
     /**
-     * Returns true if the block may contain no data. It does not guarantee
+     * Returns TRUE if the block may contain no data. It does not guarantee
      * that it really contains any no data. It can be used to speed up processing.
      * Not the difference between this method and hasNoDataValue().
-     * \returns true if the block may contain no data */
-    bool hasNoData() const;
+     * \returns TRUE if the block may contain no data
+    */
+    bool hasNoData() const SIP_HOLDGIL
+    {
+      return mHasNoDataValue || mNoDataBitmap;
+    }
 
     /**
      * Sets cell value that will be considered as "no data".
-     * \since QGIS 3.0
      * \see noDataValue(), hasNoDataValue(), resetNoDataValue()
+     * \since QGIS 3.0
      */
-    void setNoDataValue( double noDataValue );
+    void setNoDataValue( double noDataValue ) SIP_HOLDGIL;
 
     /**
      * Reset no data value: if there was a no data value previously set,
      * it will be discarded.
-     * \since QGIS 3.0
      * \see noDataValue(), hasNoDataValue(), setNoDataValue()
+     * \since QGIS 3.0
      */
-    void resetNoDataValue();
+    void resetNoDataValue() SIP_HOLDGIL;
 
     /**
-     * Return no data value. If the block does not have a no data value the
+     * Returns no data value. If the block does not have a no data value the
      *  returned value is undefined.
      * \returns No data value
      * \see hasNoDataValue(), setNoDataValue(), resetNoDataValue()
      */
-    double noDataValue() const { return mNoDataValue; }
+    double noDataValue() const SIP_HOLDGIL { return mNoDataValue; }
 
     /**
-     * Get byte array representing a value.
+     * Gets byte array representing a value.
      * \param dataType data type
      * \param value value
-     * \returns byte array representing the value */
+     * \returns byte array representing the value
+    */
     static QByteArray valueBytes( Qgis::DataType dataType, double value );
 
     /**
-     * \brief Read a single value if type of block is numeric. If type is color,
-     *  returned value is undefined.
-     *  \param row row index
-     *  \param column column index
-     *  \returns value */
-    double value( int row, int column ) const;
+     * Read a single value if type of block is numeric. If type is color,
+     * returned value is undefined.
+     * \param row row index
+     * \param column column index
+     * \returns value
+     * \see valueAndNoData()
+    */
+    double value( int row, int column ) const SIP_HOLDGIL
+    {
+      return value( static_cast< qgssize >( row ) * mWidth + column );
+    }
 
     /**
-     * \brief Read a single value if type of block is numeric. If type is color,
-     *  returned value is undefined.
-     *  \param index data matrix index (long type in Python)
-     *  \returns value */
-    double value( qgssize index ) const;
+     * Reads a single value from the pixel at \a row and \a column, if type of block is numeric. If type is color,
+     * returned value is undefined.
+     *
+     * Additionally, the \a isNoData argument will be set to TRUE if the pixel represents a nodata value. This method
+     * is more efficient then calling isNoData() and value() separately.
+     *
+     * \note Not available in Python bindings
+     * \see value()
+     * \see isNoData()
+     * \since QGIS 3.6
+     */
+    double valueAndNoData( int row, int column, bool &isNoData ) const SIP_SKIP
+    {
+      return valueAndNoData( static_cast< qgssize >( row ) * mWidth + column, isNoData );
+    }
+
+    /**
+     * Reads a single value if type of block is numeric. If type is color,
+     * returned value is undefined.
+     * \param index data matrix index (long type in Python)
+     * \returns value
+     * \see valueAndNoData()
+    */
+    inline double value( qgssize index ) const SIP_HOLDGIL;
+
+    /**
+     * Reads a single value from the pixel at the specified data matrix \a index, if type of block is numeric. If type is color,
+     * returned value is undefined.
+     *
+     * Additionally, the \a isNoData argument will be set to TRUE if the pixel represents a nodata value. This method
+     * is more efficient then calling isNoData() and value() separately.
+     *
+     * \note Not available in Python bindings
+     * \see value()
+     * \see isNoData()
+     * \since QGIS 3.6
+     */
+    inline double valueAndNoData( qgssize index, bool &isNoData ) const SIP_SKIP;
+
+    /**
+     * Gives direct access to the raster block data.
+     * The data type of the block must be Qgis::DataType::Byte otherwise it returns NULLPTR.
+     * Useful for most efficient read access.
+     * \note not available in Python bindings
+     * \since QGIS 3.4
+     */
+    const quint8 *byteData() const SIP_SKIP
+    {
+      if ( mDataType != Qgis::DataType::Byte )
+        return nullptr;
+      return static_cast< const quint8 * >( mData );
+    }
 
     /**
      * \brief Read a single color
      *  \param row row index
      *  \param column column index
-     *  \returns color */
-    QRgb color( int row, int column ) const;
+     *  \returns color
+    */
+    QRgb color( int row, int column ) const SIP_HOLDGIL
+    {
+      if ( !mImage ) return NO_DATA_COLOR;
+
+      return mImage->pixel( column, row );
+    }
 
     /**
      * \brief Read a single value
      *  \param index data matrix index (long type in Python)
-     *  \returns color */
-    QRgb color( qgssize index ) const;
+     *  \returns color
+    */
+    QRgb color( qgssize index ) const SIP_HOLDGIL
+    {
+      const int row = static_cast< int >( std::floor( static_cast< double >( index ) / mWidth ) );
+      const int column = index % mWidth;
+      return color( row, column );
+    }
 
     /**
-     * \brief Check if value at position is no data
-     *  \param row row index
-     *  \param column column index
-     *  \returns true if value is no data */
-    bool isNoData( int row, int column );
+     * Checks if value at position is no data
+     * \param row row index
+     * \param column column index
+     * \returns TRUE if value is no data
+     * \see valueAndNoData()
+    */
+    bool isNoData( int row, int column ) const SIP_HOLDGIL
+    {
+      return isNoData( static_cast< qgssize >( row ) * mWidth + column );
+    }
 
     /**
-     * \brief Check if value at position is no data
-     *  \param index data matrix index (long type in Python)
-     *  \returns true if value is no data */
-    bool isNoData( qgssize index );
+     * Check if value at position is no data
+     * \param row row index
+     * \param column column index
+     * \returns TRUE if value is no data
+     * \see valueAndNoData()
+    */
+    bool isNoData( qgssize row, qgssize column ) const SIP_HOLDGIL
+    {
+      return isNoData( row * static_cast< qgssize >( mWidth ) + column );
+    }
+
+    /**
+     * Check if value at position is no data
+     * \param index data matrix index (long type in Python)
+     * \returns TRUE if value is no data
+     * \see valueAndNoData()
+    */
+    bool isNoData( qgssize index ) const SIP_HOLDGIL
+    {
+      if ( !mHasNoDataValue && !mNoDataBitmap )
+        return false;
+      if ( index >= static_cast< qgssize >( mWidth )*mHeight )
+      {
+        QgsDebugMsg( QStringLiteral( "Index %1 out of range (%2 x %3)" ).arg( index ).arg( mWidth ).arg( mHeight ) );
+        return true; // we consider no data if outside
+      }
+      if ( mHasNoDataValue )
+      {
+        const double value = readValue( mData, mDataType, index );
+        return isNoDataValue( value );
+      }
+      // use no data bitmap
+      if ( !mNoDataBitmap )
+      {
+        // no data are not defined
+        return false;
+      }
+      // TODO: optimize
+      const int row = static_cast< int >( index ) / mWidth;
+      const int column = index % mWidth;
+      const qgssize byte = static_cast< qgssize >( row ) * mNoDataBitmapWidth + column / 8;
+      const int bit = column % 8;
+      const int mask = 0x80 >> bit;
+      //int x = mNoDataBitmap[byte] & mask;
+      //QgsDebugMsg ( QString("byte = %1 bit = %2 mask = %3 nodata = %4 is nodata = %5").arg(byte).arg(bit).arg(mask, 0, 2 ).arg( x, 0, 2 ).arg( (bool)(x) ) );
+      return mNoDataBitmap[byte] & mask;
+    }
 
     /**
      * \brief Set value on position
      *  \param row row index
      *  \param column column index
      *  \param value the value to be set
-     *  \returns true on success */
-    bool setValue( int row, int column, double value );
+     *  \returns TRUE on success
+    */
+    bool setValue( int row, int column, double value ) SIP_HOLDGIL
+    {
+      return setValue( static_cast< qgssize >( row ) * mWidth + column, value );
+    }
 
     /**
      * \brief Set value on index (indexed line by line)
      *  \param index data matrix index (long type in Python)
      *  \param value the value to be set
-     *  \returns true on success */
-    bool setValue( qgssize index, double value );
+     *  \returns TRUE on success
+    */
+    bool setValue( qgssize index, double value ) SIP_HOLDGIL
+    {
+      if ( !mData )
+      {
+        QgsDebugMsg( QStringLiteral( "Data block not allocated" ) );
+        return false;
+      }
+      if ( index >= static_cast< qgssize >( mWidth ) *mHeight )
+      {
+        QgsDebugMsg( QStringLiteral( "Index %1 out of range (%2 x %3)" ).arg( index ).arg( mWidth ).arg( mHeight ) );
+        return false;
+      }
+      writeValue( mData, mDataType, index, value );
+      return true;
+    }
 
     /**
      * \brief Set color on position
      *  \param row row index
      *  \param column column index
      *  \param color the color to be set, QRgb value
-     *  \returns true on success */
-    bool setColor( int row, int column, QRgb color );
+     *  \returns TRUE on success
+    */
+    bool setColor( int row, int column, QRgb color ) SIP_HOLDGIL
+    {
+      return setColor( static_cast< qgssize >( row ) * mWidth + column, color );
+    }
 
     /**
      * \brief Set color on index (indexed line by line)
      *  \param index data matrix index (long type in Python)
      *  \param color the color to be set, QRgb value
-     *  \returns true on success */
-    bool setColor( qgssize index, QRgb color );
+     *  \returns TRUE on success
+    */
+    bool setColor( qgssize index, QRgb color ) SIP_HOLDGIL
+    {
+      if ( !mImage )
+      {
+        QgsDebugMsg( QStringLiteral( "Image not allocated" ) );
+        return false;
+      }
+
+      if ( index >= static_cast< qgssize >( mImage->width() ) * mImage->height() )
+      {
+        QgsDebugMsg( QStringLiteral( "index %1 out of range" ).arg( index ) );
+        return false;
+      }
+
+      // setPixel() is slow, see Qt doc -> use direct access
+      QRgb *bits = reinterpret_cast< QRgb * >( mImage->bits() );
+      bits[index] = color;
+      return true;
+    }
+
+    /**
+     * Gives direct read/write access to the raster RGB data.
+     * The data type of the block must be Qgis::DataType::ARGB32 or Qgis::DataType::ARGB32_Premultiplied otherwise it returns NULLPTR.
+     * Useful for most efficient read/write access to RGB blocks.
+     * \note not available in Python bindings
+     * \since QGIS 3.4
+     */
+    QRgb *colorData() SIP_SKIP
+    {
+      if ( !mImage )
+        return nullptr;
+      return reinterpret_cast< QRgb * >( mImage->bits() );
+    }
 
     /**
      * \brief Set no data on pixel
      *  \param row row index
      *  \param column column index
-     *  \returns true on success */
-    bool setIsNoData( int row, int column );
+     *  \returns TRUE on success
+    */
+    bool setIsNoData( int row, int column ) SIP_HOLDGIL
+    {
+      return setIsNoData( static_cast< qgssize >( row ) * mWidth + column );
+    }
 
     /**
      * \brief Set no data on pixel
      *  \param index data matrix index (long type in Python)
-     *  \returns true on success */
-    bool setIsNoData( qgssize index );
+     *  \returns TRUE on success
+    */
+    bool setIsNoData( qgssize index ) SIP_HOLDGIL
+    {
+      if ( mHasNoDataValue )
+      {
+        return setValue( index, mNoDataValue );
+      }
+      else
+      {
+        if ( !mNoDataBitmap )
+        {
+          if ( !createNoDataBitmap() )
+          {
+            return false;
+          }
+        }
+        // TODO: optimize
+        const int row = static_cast< int >( index ) / mWidth;
+        const int column = index % mWidth;
+        const qgssize byte = static_cast< qgssize >( row ) * mNoDataBitmapWidth + column / 8;
+        const int bit = column % 8;
+        const int nodata = 0x80 >> bit;
+        //QgsDebugMsg ( QString("set byte = %1 bit = %2 no data by %3").arg(byte).arg(bit).arg(nodata, 0,2 ) );
+        mNoDataBitmap[byte] = mNoDataBitmap[byte] | nodata;
+        return true;
+      }
+    }
 
     /**
      * \brief Set the whole block to no data
-     *  \returns true on success */
+     *  \returns TRUE on success
+    */
     bool setIsNoData();
 
     /**
      * \brief Set the whole block to no data except specified rectangle
-     *  \returns true on success */
+     *  \returns TRUE on success
+    */
     bool setIsNoDataExcept( QRect exceptRect );
 
     /**
@@ -276,8 +485,12 @@ class CORE_EXPORT QgsRasterBlock
      * method. This method has no effect for raster blocks with an explicit no data value set.
      *  \param row row index
      *  \param column column index
-     *  \since QGIS 2.10 */
-    void setIsData( int row, int column );
+     *  \since QGIS 2.10
+    */
+    void setIsData( int row, int column ) SIP_HOLDGIL
+    {
+      setIsData( static_cast< qgssize >( row )*mWidth + column );
+    }
 
     /**
      * \brief Remove no data flag on pixel. If the raster block does not have an explicit
@@ -285,11 +498,32 @@ class CORE_EXPORT QgsRasterBlock
      * In this case it is possible to reset a pixel to flag it as having valid data using this
      * method. This method has no effect for raster blocks with an explicit no data value set.
      *  \param index data matrix index (long type in Python)
-     *  \since QGIS 2.10 */
-    void setIsData( qgssize index );
+     *  \since QGIS 2.10
+    */
+    void setIsData( qgssize index ) SIP_HOLDGIL
+    {
+      if ( mHasNoDataValue )
+      {
+        //no data value set, so mNoDataBitmap is not being used
+        return;
+      }
+
+      if ( !mNoDataBitmap )
+      {
+        return;
+      }
+
+      // TODO: optimize
+      const int row = static_cast< int >( index ) / mWidth;
+      const int column = index % mWidth;
+      const qgssize byte = static_cast< qgssize >( row ) * mNoDataBitmapWidth + column / 8;
+      const int bit = column % 8;
+      const int nodata = 0x80 >> bit;
+      mNoDataBitmap[byte] = mNoDataBitmap[byte] & ~nodata;
+    }
 
     /**
-     * Get access to raw data.
+     * Gets access to raw data.
      * The returned QByteArray instance is not a copy of the data: it only refers to the array
      * owned by the QgsRasterBlock, therefore it is only valid while the QgsRasterBlock object
      * still exists. Writing to the returned QByteArray will not affect the original data:
@@ -311,26 +545,23 @@ class CORE_EXPORT QgsRasterBlock
     void setData( const QByteArray &data, int offset = 0 );
 
     /**
-     * \brief Get pointer to data
-     *  \param row row index
-     *  \param column column index
-     *  \returns pointer to data
-     *  \note not available in Python bindings
+     * Returns a pointer to block data.
+     * \param row row index
+     * \param column column index
+     * \note not available in Python bindings
      */
     char *bits( int row, int column ) SIP_SKIP;
 
     /**
-     * \brief Get pointer to data
-     *  \param index data matrix index (long type in Python)
-     *  \returns pointer to data
-     *  \note not available in Python bindings
+     * Returns a pointer to block data.
+     * \param index data matrix index (long type in Python)
+     * \note not available in Python bindings
      */
     char *bits( qgssize index ) SIP_SKIP;
 
     /**
-     * \brief Get pointer to data
-     *  \returns pointer to data
-     *  \note not available in Python bindings
+     * Returns a pointer to block data.
+     * \note not available in Python bindings
      */
     char *bits() SIP_SKIP;
 
@@ -338,7 +569,8 @@ class CORE_EXPORT QgsRasterBlock
      * \brief Print double value with all necessary significant digits.
      *         It is ensured that conversion back to double gives the same number.
      *  \param value the value to be printed
-     *  \returns string representing the value*/
+     *  \returns string representing the value
+     */
     static QString printValue( double value );
 
     /**
@@ -346,26 +578,27 @@ class CORE_EXPORT QgsRasterBlock
      *         It is ensured that conversion back to float gives the same number.
      *  \param value the value to be printed
      *  \returns string representing the value
-     * \since QGIS 2.16
      * \note not available in Python bindings
+     * \since QGIS 2.16
      */
     static QString printValue( float value ) SIP_SKIP;
 
     /**
      * \brief Convert data to different type.
      *  \param destDataType dest data type
-     *  \returns true on success */
+     *  \returns TRUE on success
+    */
     bool convert( Qgis::DataType destDataType );
 
     /**
-     * \brief Get image if type is color.
-    *   \returns image */
+     * Returns an image containing the block data, if the block's data type is color.
+     */
     QImage image() const;
 
     /**
-     * \brief set image.
-     *  \param image image
-     *  \returns true on success */
+     * Sets the block data via an \a image.
+     * \returns TRUE on success
+    */
     bool setImage( const QImage *image );
 
     //! \note not available in Python bindings
@@ -382,10 +615,10 @@ class CORE_EXPORT QgsRasterBlock
     */
     void applyScaleOffset( double scale, double offset );
 
-    //! \brief Get error
+    //! Returns the last error
     QgsError error() const { return mError; }
 
-    //! \brief Set error
+    //! Sets the last error
     void setError( const QgsError &error ) { mError = error;}
 
     QString toString() const;
@@ -407,14 +640,14 @@ class CORE_EXPORT QgsRasterBlock
      * \see height
      * \since QGIS 2.10
      */
-    int width() const { return mWidth; }
+    int width() const SIP_HOLDGIL { return mWidth; }
 
     /**
      * Returns the height (number of rows) of the raster block.
      * \see width
      * \since QGIS 2.10
      */
-    int height() const { return mHeight; }
+    int height() const SIP_HOLDGIL { return mHeight; }
 
   private:
     static QImage::Format imageFormat( Qgis::DataType dataType );
@@ -424,18 +657,28 @@ class CORE_EXPORT QgsRasterBlock
      * Test if value is nodata comparing to noDataValue
      * \param value tested value
      * \param noDataValue no data value
-     * \returns true if value is nodata */
-    static bool isNoDataValue( double value, double noDataValue );
+     * \returns TRUE if value is nodata
+    */
+    static bool isNoDataValue( double value, double noDataValue )
+    {
+      // TODO: optimize no data value test by memcmp()
+      // More precise would be std::isnan(value) && std::isnan(noDataValue(bandNo)), but probably
+      // not important and slower
+      return std::isnan( value ) ||
+             qgsDoubleNear( value, noDataValue );
+    }
 
     /**
      * Test if value is nodata for specific band
      * \param value tested value
-     * \returns true if value is nodata */
-    bool isNoDataValue( double value ) const;
+     * \returns TRUE if value is nodata
+    */
+    inline bool isNoDataValue( double value ) const;
 
     /**
      * Allocate no data bitmap
-     *  \returns true on success */
+     *  \returns TRUE on success
+    */
     bool createNoDataBitmap();
 
     /**
@@ -445,14 +688,15 @@ class CORE_EXPORT QgsRasterBlock
      *  \param srcDataType source data type
      *  \param destDataType dest data type
      *  \param size block size (width * height)
-     *  \returns block of data in destDataType */
+     *  \returns block of data in destDataType
+    */
     static void *convert( void *srcData, Qgis::DataType srcDataType, Qgis::DataType destDataType, qgssize size );
 
     // Valid
     bool mValid = true;
 
     // Data type
-    Qgis::DataType mDataType = Qgis::UnknownDataType;
+    Qgis::DataType mDataType = Qgis::DataType::UnknownDataType;
 
     // Data type size in bytes, to make bits() fast
     int mTypeSize = 0;
@@ -502,29 +746,22 @@ inline double QgsRasterBlock::readValue( void *data, Qgis::DataType type, qgssiz
 
   switch ( type )
   {
-    case Qgis::Byte:
+    case Qgis::DataType::Byte:
       return static_cast< double >( ( static_cast< quint8 * >( data ) )[index] );
-      break;
-    case Qgis::UInt16:
+    case Qgis::DataType::UInt16:
       return static_cast< double >( ( static_cast< quint16 * >( data ) )[index] );
-      break;
-    case Qgis::Int16:
+    case Qgis::DataType::Int16:
       return static_cast< double >( ( static_cast< qint16 * >( data ) )[index] );
-      break;
-    case Qgis::UInt32:
+    case Qgis::DataType::UInt32:
       return static_cast< double >( ( static_cast< quint32 * >( data ) )[index] );
-      break;
-    case Qgis::Int32:
+    case Qgis::DataType::Int32:
       return static_cast< double >( ( static_cast< qint32 * >( data ) )[index] );
-      break;
-    case Qgis::Float32:
+    case Qgis::DataType::Float32:
       return static_cast< double >( ( static_cast< float * >( data ) )[index] );
-      break;
-    case Qgis::Float64:
+    case Qgis::DataType::Float64:
       return static_cast< double >( ( static_cast< double * >( data ) )[index] );
-      break;
     default:
-      QgsDebugMsg( QString( "Data type %1 is not supported" ).arg( type ) );
+      QgsDebugMsg( QStringLiteral( "Data type %1 is not supported" ).arg( qgsEnumValueToKey< Qgis::DataType >( type ) ) );
       break;
   }
 
@@ -537,29 +774,29 @@ inline void QgsRasterBlock::writeValue( void *data, Qgis::DataType type, qgssize
 
   switch ( type )
   {
-    case Qgis::Byte:
+    case Qgis::DataType::Byte:
       ( static_cast< quint8 * >( data ) )[index] = static_cast< quint8 >( value );
       break;
-    case Qgis::UInt16:
+    case Qgis::DataType::UInt16:
       ( static_cast< quint16 * >( data ) )[index] = static_cast< quint16 >( value );
       break;
-    case Qgis::Int16:
+    case Qgis::DataType::Int16:
       ( static_cast< qint16 * >( data ) )[index] = static_cast< qint16 >( value );
       break;
-    case Qgis::UInt32:
+    case Qgis::DataType::UInt32:
       ( static_cast< quint32 * >( data ) )[index] = static_cast< quint32 >( value );
       break;
-    case Qgis::Int32:
+    case Qgis::DataType::Int32:
       ( static_cast< qint32 * >( data ) )[index] = static_cast< qint32 >( value );
       break;
-    case Qgis::Float32:
+    case Qgis::DataType::Float32:
       ( static_cast< float * >( data ) )[index] = static_cast< float >( value );
       break;
-    case Qgis::Float64:
+    case Qgis::DataType::Float64:
       ( static_cast< double * >( data ) )[index] = value;
       break;
     default:
-      QgsDebugMsg( QString( "Data type %1 is not supported" ).arg( type ) );
+      QgsDebugMsg( QStringLiteral( "Data type %1 is not supported" ).arg( qgsEnumValueToKey< Qgis::DataType >( type ) ) );
       break;
   }
 }
@@ -568,12 +805,53 @@ inline double QgsRasterBlock::value( qgssize index ) const SIP_SKIP
 {
   if ( !mData )
   {
-    QgsDebugMsg( "Data block not allocated" );
+    QgsDebugMsg( QStringLiteral( "Data block not allocated" ) );
     return std::numeric_limits<double>::quiet_NaN();
   }
   return readValue( mData, mDataType, index );
 }
 
+inline double QgsRasterBlock::valueAndNoData( qgssize index, bool &isNoData ) const SIP_SKIP
+{
+  if ( !mData )
+  {
+    QgsDebugMsg( QStringLiteral( "Data block not allocated" ) );
+    isNoData = true;
+    return std::numeric_limits<double>::quiet_NaN();
+  }
+  if ( index >= static_cast< qgssize >( mWidth )*mHeight )
+  {
+    QgsDebugMsg( QStringLiteral( "Index %1 out of range (%2 x %3)" ).arg( index ).arg( mWidth ).arg( mHeight ) );
+    isNoData = true; // we consider no data if outside
+    return std::numeric_limits<double>::quiet_NaN();
+  }
+
+  const double val = readValue( mData, mDataType, index );
+
+  if ( !mHasNoDataValue && !mNoDataBitmap )
+  {
+    isNoData = false;
+    return val;
+  }
+
+  if ( mHasNoDataValue )
+  {
+    isNoData = isNoDataValue( val );
+    return val;
+  }
+  // use no data bitmap
+  if ( !mNoDataBitmap )
+  {
+    // no data are not defined
+    isNoData = false;
+    return val;
+  }
+
+  // no data is a bitmap
+  isNoData = QgsRasterBlock::isNoData( index );
+  return val;
+}
+
 inline bool QgsRasterBlock::isNoDataValue( double value ) const SIP_SKIP
 {
   return std::isnan( value ) || qgsDoubleNear( value, mNoDataValue );
