@@ -21,12 +21,7 @@ __author__ = 'Matteo Ghetta'
 __date__ = 'March 2017'
 __copyright__ = '(C) 2017, Matteo Ghetta'
 
-# This will get replaced with a git SHA1 when you do a git archive
-
-__revision__ = 'bc43194061452b37c8a22edcb7b380055a0c4236'
-
-import plotly as plt
-import plotly.graph_objs as go
+import warnings
 
 from qgis.core import (QgsProcessingException,
                        QgsProcessingParameterFeatureSource,
@@ -37,9 +32,10 @@ from qgis.core import (QgsProcessingException,
 from processing.algs.qgis.QgisAlgorithm import QgisAlgorithm
 from processing.tools import vector
 
+from qgis.PyQt.QtCore import QCoreApplication
+
 
 class BoxPlot(QgisAlgorithm):
-
     INPUT = 'INPUT'
     OUTPUT = 'OUTPUT'
     NAME_FIELD = 'NAME_FIELD'
@@ -47,10 +43,10 @@ class BoxPlot(QgisAlgorithm):
     MSD = 'MSD'
 
     def group(self):
-        return self.tr('Graphics')
+        return self.tr('Plots')
 
     def groupId(self):
-        return 'graphics'
+        return 'plots'
 
     def __init__(self):
         super().__init__()
@@ -84,6 +80,16 @@ class BoxPlot(QgisAlgorithm):
         return self.tr('Box plot')
 
     def processAlgorithm(self, parameters, context, feedback):
+        try:
+            # importing plotly throws Python warnings from within the library - filter these out
+            with warnings.catch_warnings():
+                warnings.filterwarnings("ignore", category=ResourceWarning)
+                warnings.filterwarnings("ignore", category=ImportWarning)
+                import plotly as plt
+                import plotly.graph_objs as go
+        except ImportError:
+            raise QgsProcessingException(QCoreApplication.translate('BoxPlot', 'This algorithm requires the Python “plotly” library. Please install this library and try again.'))
+
         source = self.parameterAsSource(parameters, self.INPUT, context)
         if source is None:
             raise QgsProcessingException(self.invalidSourceError(parameters, self.INPUT))
