@@ -68,18 +68,15 @@ class ParameterReliefColors(QgsProcessingParameterDefinition):
 
     @staticmethod
     def colorsToString(colors):
-        s = ''
-        for c in colors:
-            s += '{:f}, {:f}, {:d}, {:d}, {:d};'.format(c[0],
-                                                        c[1],
-                                                        c[2],
-                                                        c[3],
-                                                        c[4])
-        return s[:-1]
+        return ';'.join('{:f}, {:f}, {:d}, {:d}, {:d}'.format(c[0],
+                                                              c[1],
+                                                              c[2],
+                                                              c[3],
+                                                              c[4])
+                        for c in colors)
 
 
 class Relief(QgisAlgorithm):
-
     INPUT = 'INPUT'
     Z_FACTOR = 'Z_FACTOR'
     AUTO_COLORS = 'AUTO_COLORS'
@@ -157,6 +154,20 @@ class Relief(QgisAlgorithm):
         relief.setZFactor(zFactor)
         if frequencyDistribution:
             relief.exportFrequencyDistributionToCsv(frequencyDistribution)
-        relief.processRaster(feedback)
+        res = relief.processRaster(feedback)
+        if res == 1:
+            raise QgsProcessingException(self.tr('Can not open input file.'))
+        elif res == 2:
+            raise QgsProcessingException(self.tr('Can not get GDAL driver for output file.'))
+        elif res == 3:
+            raise QgsProcessingException(self.tr('Can not create output file.'))
+        elif res == 4:
+            raise QgsProcessingException(self.tr('Can not get input band.'))
+        elif res == 5:
+            raise QgsProcessingException(self.tr('Can not create output bands.'))
+        elif res == 6:
+            raise QgsProcessingException(self.tr('Output raster size is too small (at least 3 rows needed).'))
+        elif res == 7:
+            feedback.pushInfo(self.tr('Canceled.'))
 
         return {self.OUTPUT: outputFile, self.FREQUENCY_DISTRIBUTION: frequencyDistribution}
