@@ -26,15 +26,26 @@
 import os
 import json
 import zipfile
+from functools import partial
 
+from qgis.PyQt import sip
 from qgis.PyQt.QtCore import Qt, QObject, QDir, QUrl, QFileInfo, QFile
-from qgis.PyQt.QtWidgets import QApplication, QDialog, QDialogButtonBox, QFrame, QMessageBox, QLabel, QVBoxLayout
+from qgis.PyQt.QtWidgets import (
+    QApplication,
+    QDialog,
+    QDialogButtonBox,
+    QFrame,
+    QMessageBox,
+    QLabel,
+    QVBoxLayout,
+    QPushButton
+)
 from qgis.PyQt.QtNetwork import QNetworkRequest
 
 import qgis
 from qgis.core import Qgis, QgsApplication, QgsNetworkAccessManager, QgsSettings, QgsNetworkRequestParameters
 from qgis.gui import QgsMessageBar, QgsPasswordLineEdit, QgsHelp
-from qgis.utils import (iface, startPlugin, unloadPlugin, loadPlugin,
+from qgis.utils import (iface, startPlugin, unloadPlugin, loadPlugin, OverrideCursor,
                         reloadPlugin, updateAvailablePlugins, plugins_metadata_parser)
 from .installer_data import (repositories, plugins, officialRepo,
                              settingsGroup, reposGroup, removeDir)
@@ -61,8 +72,6 @@ class QgsPluginInstaller(QObject):
 
     """ The main class for managing the plugin installer stuff"""
 
-    statusLabel = None
-
     # ----------------------------------------- #
     def __init__(self):
         """ Initialize data objects, starts fetching if appropriate, and warn about/removes obsolete plugins """
@@ -71,11 +80,10 @@ class QgsPluginInstaller(QObject):
         repositories.load()
         plugins.getAllInstalled()
 
+        self.message_bar_widget = None
+
         if repositories.checkingOnStart() and repositories.timeForChecking() and repositories.allEnabled():
             # start fetching repositories
-            self.statusLabel = QLabel(iface.mainWindow().statusBar())
-            iface.mainWindow().statusBar().addPermanentWidget(self.statusLabel)
-            self.statusLabel.linkActivated.connect(self.showPluginManagerWhenReady)
             repositories.checkingDone.connect(self.checkingDone)
             for key in repositories.allEnabled():
                 repositories.requestFetching(key)
@@ -95,11 +103,13 @@ class QgsPluginInstaller(QObject):
             msg.setText("%s <b>%s</b><br/><br/>%s" % (self.tr("Obsolete plugin:"), plugin["name"], self.tr("QGIS has detected an obsolete plugin that masks its more recent version shipped with this copy of QGIS. This is likely due to files associated with a previous installation of QGIS. Do you want to remove the old plugin right now and unmask the more recent version?")))
             msg.exec_()
             if not msg.result():
+                settings = QgsSettings()
+                plugin_is_active = settings.value("/PythonPlugins/" + key, False, type=bool)
+
                 # uninstall the update, update utils and reload if enabled
                 self.uninstallPlugin(key, quiet=True)
                 updateAvailablePlugins()
-                settings = QgsSettings()
-                if settings.value("/PythonPlugins/" + key, False, type=bool):
+                if plugin_is_active:
                     settings.setValue("/PythonPlugins/watchDog/" + key, True)
                     loadPlugin(key)
                     startPlugin(key)
@@ -110,25 +120,22 @@ class QgsPluginInstaller(QObject):
         """ Fetch plugins from all enabled repositories."""
         """  reloadMode = true:  Fully refresh data from QgsSettings to mRepositories  """
         """  reloadMode = false: Fetch unready repositories only """
-        QApplication.setOverrideCursor(Qt.WaitCursor)
+        with OverrideCursor(Qt.WaitCursor):
+            if reloadMode:
+                repositories.load()
+                plugins.clearRepoCache()
+                plugins.getAllInstalled()
 
-        if reloadMode:
-            repositories.load()
-            plugins.clearRepoCache()
-            plugins.getAllInstalled()
+            for key in repositories.allEnabled():
+                if reloadMode or repositories.all()[key]["state"] == 3:  # if state = 3 (error or not fetched yet), try to fetch once again
+                    repositories.requestFetching(key, force_reload=reloadMode)
 
-        for key in repositories.allEnabled():
-            if reloadMode or repositories.all()[key]["state"] == 3:  # if state = 3 (error or not fetched yet), try to fetch once again
-                repositories.requestFetching(key, force_reload=reloadMode)
-
-        if repositories.fetchingInProgress():
-            fetchDlg = QgsPluginInstallerFetchingDialog(iface.mainWindow())
-            fetchDlg.exec_()
-            del fetchDlg
-            for key in repositories.all():
-                repositories.killConnection(key)
-
-        QApplication.restoreOverrideCursor()
+            if repositories.fetchingInProgress():
+                fetchDlg = QgsPluginInstallerFetchingDialog(iface.mainWindow())
+                fetchDlg.exec_()
+                del fetchDlg
+                for key in repositories.all():
+                    repositories.killConnection(key)
 
         # display error messages for every unavailable repository, unless Shift pressed nor all repositories are unavailable
         keepQuiet = QgsApplication.keyboardModifiers() == Qt.KeyboardModifiers(Qt.ShiftModifier)
@@ -143,35 +150,36 @@ class QgsPluginInstaller(QObject):
 
     # ----------------------------------------- #
     def checkingDone(self):
-        """ Remove the "Looking for new plugins..." label and display a notification instead if any updates or news available """
-        if not self.statusLabel:
-            # only proceed if the label is present
-            return
+        """ Remove the "Looking for new plugins..." label and display a notification instead if any updates available """
+
         # rebuild plugins cache
         plugins.rebuild()
         # look for news in the repositories
         plugins.markNews()
-        status = ""
-        icon = ""
-        # first check for news
-        for key in plugins.all():
-            if plugins.all()[key]["status"] == "new":
-                status = self.tr("There is a new plugin available")
-                icon = "pluginNew.svg"
-                tabIndex = 4  # PLUGMAN_TAB_NEW
+        status = None
         # then check for updates (and eventually overwrite status)
-        for key in plugins.all():
-            if plugins.all()[key]["status"] == "upgradeable":
-                status = self.tr("There is a plugin update available")
-                icon = "pluginUpgrade.svg"
-                tabIndex = 3  # PLUGMAN_TAB_UPGRADEABLE
+        updatable_count = 0
+        updatable_plugin_name = None
+        for _, properties in plugins.all().items():
+            if properties["status"] == "upgradeable":
+                updatable_count += 1
+                updatable_plugin_name = properties["name"]
+
+        if updatable_count:
+            if updatable_count > 1:
+                status = self.tr("Multiple plugin updates are available")
+            else:
+                status = self.tr("An update to the {} plugin is available").format(updatable_plugin_name)
+            tabIndex = 3  # PLUGMAN_TAB_UPGRADEABLE
+
         # finally set the notify label
         if status:
-            self.statusLabel.setText(u'<a href="%d"><img src="qrc:/images/themes/default/%s"></a>' % (tabIndex, icon))
-            self.statusLabel.setToolTip(status)
-        else:
-            iface.mainWindow().statusBar().removeWidget(self.statusLabel)
-            self.statusLabel = None
+            bar = iface.messageBar()
+            self.message_bar_widget = bar.createMessage('', status)
+            update_button = QPushButton("Install Updates…")
+            update_button.pressed.connect(partial(self.showPluginManagerWhenReady, tabIndex))
+            self.message_bar_widget.layout().addWidget(update_button)
+            bar.pushWidget(self.message_bar_widget, Qgis.Info)
 
     # ----------------------------------------- #
     def exportRepositoriesToManager(self):
@@ -259,9 +267,10 @@ class QgsPluginInstaller(QObject):
     def showPluginManagerWhenReady(self, * params):
         """ Open the plugin manager window. If fetching is still in progress, it shows the progress window first """
         """ Optionally pass the index of tab to be opened in params """
-        if self.statusLabel:
-            iface.mainWindow().statusBar().removeWidget(self.statusLabel)
-            self.statusLabel = None
+        if self.message_bar_widget:
+            if not sip.isdeleted(self.message_bar_widget):
+                iface.messageBar().popWidget(self.message_bar_widget)
+            self.message_bar_widget = None
 
         self.fetchAvailablePlugins(reloadMode=False)
         self.exportRepositoriesToManager()
@@ -323,11 +332,10 @@ class QgsPluginInstaller(QObject):
                     "the list of installed plugins. You should find the plugin there, but it's not possible to "
                     "determine which of them it is and it's also not possible to inform you about available updates. "
                     "Please contact the plugin author and submit this issue.").format(plugin_path))
-            QApplication.setOverrideCursor(Qt.WaitCursor)
-            plugins.getAllInstalled()
-            plugins.rebuild()
-            self.exportPluginsToManager()
-            QApplication.restoreOverrideCursor()
+            with OverrideCursor(Qt.WaitCursor):
+                plugins.getAllInstalled()
+                plugins.rebuild()
+                self.exportPluginsToManager()
         else:
             QApplication.setOverrideCursor(Qt.WaitCursor)
             # update the list of plugins in plugin handling routines
@@ -410,7 +418,7 @@ class QgsPluginInstaller(QObject):
             return
         if not quiet:
             warning = self.tr("Are you sure you want to uninstall the following plugin?") + "\n(" + plugin["name"] + ")"
-            if plugin["status"] == "orphan" and not plugin["error"]:
+            if plugin["status"] == "orphan" and plugin["status_exp"] == "orphan" and not plugin["error"]:
                 warning += "\n\n" + self.tr("Warning: this plugin isn't available in any accessible repository!")
             if QMessageBox.warning(iface.mainWindow(), self.tr("QGIS Python Plugin Installer"), warning, QMessageBox.Yes, QMessageBox.No) == QMessageBox.No:
                 return
@@ -452,6 +460,9 @@ class QgsPluginInstaller(QObject):
             QApplication.restoreOverrideCursor()
             iface.pluginManagerInterface().pushMessage(self.tr("Plugin uninstalled successfully"), Qgis.Info)
 
+            settings = QgsSettings()
+            settings.remove("/PythonPlugins/" + key)
+
     # ----------------------------------------- #
     def addRepository(self):
         """ add new repository connection """
@@ -523,7 +534,7 @@ class QgsPluginInstaller(QObject):
         self.reloadAndExportData()
 
     # ----------------------------------------- #
-    def deleteRepository(self, reposName):
+    def deleteRepository(self, reposName: str):
         """ delete repository connection """
         if not reposName:
             return
@@ -570,8 +581,13 @@ class QgsPluginInstaller(QObject):
         settings.setValue(settingsGroup + '/lastZipDirectory',
                           QFileInfo(filePath).absoluteDir().absolutePath())
 
+        pluginName = None
         with zipfile.ZipFile(filePath, 'r') as zf:
-            pluginName = os.path.split(zf.namelist()[0])[0]
+            # search for metadata.txt. In case of multiple files, we can assume that
+            # the shortest path relates <pluginname>/metadata.txt
+            metadatafiles = sorted(f for f in zf.namelist() if f.endswith('metadata.txt'))
+            if len(metadatafiles) > 0:
+                pluginName = os.path.split(metadatafiles[0])[0]
 
         pluginFileName = os.path.splitext(os.path.basename(filePath))[0]
 
@@ -639,22 +655,22 @@ class QgsPluginInstaller(QObject):
                     keepTrying = False
 
         if success:
-            updateAvailablePlugins()
-            self.processDependencies(pluginName)
-            loadPlugin(pluginName)
-            plugins.getAllInstalled()
-            plugins.rebuild()
+            with OverrideCursor(Qt.WaitCursor):
+                updateAvailablePlugins()
+                self.processDependencies(pluginName)
+                loadPlugin(pluginName)
+                plugins.getAllInstalled()
+                plugins.rebuild()
 
-            if settings.contains('/PythonPlugins/' + pluginName):
-                if settings.value('/PythonPlugins/' + pluginName, False, bool):
-                    startPlugin(pluginName)
-                    reloadPlugin(pluginName)
+                if settings.contains('/PythonPlugins/' + pluginName):  # Plugin was available?
+                    if settings.value('/PythonPlugins/' + pluginName, False, bool):  # Plugin was also active?
+                        reloadPlugin(pluginName)  # unloadPlugin + loadPlugin + startPlugin
+                    else:
+                        unloadPlugin(pluginName)
+                        loadPlugin(pluginName)
                 else:
-                    unloadPlugin(pluginName)
-                    loadPlugin(pluginName)
-            else:
-                if startPlugin(pluginName):
-                    settings.setValue('/PythonPlugins/' + pluginName, True)
+                    if startPlugin(pluginName):
+                        settings.setValue('/PythonPlugins/' + pluginName, True)
 
             self.exportPluginsToManager()
             msg = "<b>%s</b>" % self.tr("Plugin installed successfully")
@@ -676,17 +692,17 @@ class QgsPluginInstaller(QObject):
             dlg = QgsPluginDependenciesDialog(plugin_id, to_install, to_upgrade, not_found)
             if dlg.exec_() == QgsPluginDependenciesDialog.Accepted:
                 actions = dlg.actions()
-                for dependency_plugin_id, action in actions.items():
+                for dependency_plugin_id, action_data in actions.items():
                     try:
-                        self.installPlugin(dependency_plugin_id)
-                        if action == 'install':
+                        self.installPlugin(dependency_plugin_id, stable=action_data['use_stable_version'])
+                        if action_data['action'] == 'install':
                             iface.pluginManagerInterface().pushMessage(self.tr("Plugin dependency <b>%s</b> successfully installed") %
                                                                        dependency_plugin_id, Qgis.Info)
                         else:
                             iface.pluginManagerInterface().pushMessage(self.tr("Plugin dependency <b>%s</b> successfully upgraded") %
                                                                        dependency_plugin_id, Qgis.Info)
                     except Exception as ex:
-                        if action == 'install':
+                        if action_data['action'] == 'install':
                             iface.pluginManagerInterface().pushMessage(self.tr("Error installing plugin dependency <b>%s</b>: %s") %
                                                                        (dependency_plugin_id, ex), Qgis.Warning)
                         else:
