@@ -22,28 +22,26 @@
  *                                                                         *
  ***************************************************************************/
 """
+from typing import (
+    Dict,
+    Optional,
+    Any
+)
 
 from qgis.PyQt.QtCore import (pyqtSignal, QObject, QCoreApplication, QFile,
                               QDir, QDirIterator, QDate, QUrl, QFileInfo,
                               QLocale, QByteArray)
 from qgis.PyQt.QtXml import QDomDocument
 from qgis.PyQt.QtNetwork import QNetworkRequest, QNetworkReply
-from qgis.core import QgsSettings
+from qgis.core import Qgis, QgsSettings, QgsNetworkRequestParameters
 import sys
 import os
 import codecs
 import re
-try:
-    import configparser
-except ImportError:
-    import configparser as configparser
-try:
-    from importlib import reload
-except ImportError:
-    from imp import reload
+import configparser
 import qgis.utils
 from qgis.core import QgsNetworkAccessManager, QgsApplication
-from qgis.gui import QgsMessageBar
+from qgis.gui import QgsGui
 from qgis.utils import iface, plugin_paths
 from .version_compare import pyQgisVersion, compareVersions, normalizeVersion, isCompatible
 
@@ -53,7 +51,7 @@ Data structure:
 mRepositories = dict of dicts: {repoName : {"url" unicode,
                                             "enabled" bool,
                                             "valid" bool,
-                                            "Relay" Relay, # Relay object for transmitting signals from QPHttp with adding the repoName information
+                                            "Relay" Relay,  # Relay object for transmitting signals from QPHttp with adding the repoName information
                                             "Request" QNetworkRequest,
                                             "xmlData" QNetworkReply,
                                             "state" int,   (0 - disabled, 1-loading, 2-loaded ok, 3-error (to be retried), 4-rejected)
@@ -81,18 +79,26 @@ mPlugins = dict of dicts {id : {
     "installed" boolean,                        # True if installed
     "available" boolean,                        # True if available in repositories
     "status" unicode,                           # ( not installed | new ) | ( installed | upgradeable | orphan | newer )
+    "status_exp" unicode,                       # ( not installed | new ) | ( installed | upgradeable | orphan | newer )
     "error" unicode,                            # NULL | broken | incompatible | dependent
     "error_details" unicode,                    # error description
     "experimental" boolean,                     # true if experimental, false if stable
     "deprecated" boolean,                       # true if deprecated, false if actual
     "trusted" boolean,                          # true if trusted, false if not trusted
     "version_available" unicode,                # available version
+    "version_available_stable" unicode,         # available stable version
+    "version_available_experimental" unicode,   # available experimental version
     "zip_repository" unicode,                   # the remote repository id
     "download_url" unicode,                     # url for downloading the plugin
+    "download_url_stable" unicode,              # url for downloading the plugin's stable version
+    "download_url_experimental" unicode,        # url for downloading the plugin's experimental version
     "filename" unicode,                         # the zip file name to be unzipped after downloaded
     "downloads" unicode,                        # number of downloads
     "average_vote" unicode,                     # average vote
     "rating_votes" unicode,                     # number of votes
+    "create_date" unicode,                      # ISO datetime when the plugin has been created
+    "update_date" unicode,                      # ISO datetime when the plugin has been last updated
+    "plugin_dependencies" unicode,              # PIP-style comma separated list of plugin dependencies
 }}
 """
 
@@ -132,14 +138,12 @@ def removeDir(path):
     if not QDir(pluginDir).exists():
         QDir().mkpath(pluginDir)
     return result
-# --- /common functions ------------------------------------------------------------------ #
 
 
-# --- class Relay  ----------------------------------------------------------------------- #
 class Relay(QObject):
-
-    """ Relay object for transmitting signals from QPHttp with adding the repoName information """
-    # ----------------------------------------- #
+    """
+    Relay object for transmitting signals from QPHttp with adding the repoName information
+    """
     anythingChanged = pyqtSignal(str, int, int)
 
     def __init__(self, key):
@@ -149,23 +153,27 @@ class Relay(QObject):
     def stateChanged(self, state):
         self.anythingChanged.emit(self.key, state, 0)
 
-    # ----------------------------------------- #
     def dataReadProgress(self, done, total):
-        state = 4
+        state = Repositories.STATE_REJECTED
         if total > 0:
             progress = int(float(done) / float(total) * 100)
         else:
             progress = 0
         self.anythingChanged.emit(self.key, state, progress)
 
-# --- /class Relay  ---------------------------------------------------------------------- #
 
-
-# --- class Repositories ----------------------------------------------------------------- #
 class Repositories(QObject):
+    """
+    A dict-like class for handling repositories data
+    """
 
-    """ A dict-like class for handling repositories data """
-    # ----------------------------------------- #
+    STATE_DISABLED = 0
+    STATE_LOADING = 1
+    STATE_LOADED = 2
+    STATE_UNAVAILABLE = 3
+    STATE_REJECTED = 4
+
+    CHECK_ON_START_INTERVAL = 3
 
     anythingChanged = pyqtSignal(str, int, int)
     repositoryFetched = pyqtSignal(str)
@@ -173,127 +181,79 @@ class Repositories(QObject):
 
     def __init__(self):
         QObject.__init__(self)
-        self.mRepositories = {}
+        self.mRepositories: Dict[str, Dict[str, Any]] = {}
         self.httpId = {}   # {httpId : repoName}
-        self.mInspectionFilter = None
+        self.mInspectionFilter: Optional[str] = None
 
-    # ----------------------------------------- #
-    def all(self):
+    def all(self) -> Dict[str, Dict[str, Any]]:
         """ return dict of all repositories """
         return self.mRepositories
 
-    # ----------------------------------------- #
-    def allEnabled(self):
+    def allEnabled(self) -> Dict[str, Dict[str, Any]]:
         """ return dict of all enabled and valid repositories """
         if self.mInspectionFilter:
             return {self.mInspectionFilter: self.mRepositories[self.mInspectionFilter]}
 
-        repos = {}
-        for i in self.mRepositories:
-            if self.mRepositories[i]["enabled"] and self.mRepositories[i]["valid"]:
-                repos[i] = self.mRepositories[i]
-        return repos
+        return {k: v for k, v in self.mRepositories.items() if v['enabled'] and v['valid']}
 
-    # ----------------------------------------- #
-    def allUnavailable(self):
+    def allUnavailable(self) -> Dict[str, Dict[str, Any]]:
         """ return dict of all unavailable repositories """
         repos = {}
 
         if self.mInspectionFilter:
             # return the inspected repo if unavailable, otherwise empty dict
-            if self.mRepositories[self.mInspectionFilter]["state"] == 3:
+            if self.mRepositories[self.mInspectionFilter]["state"] == Repositories.STATE_UNAVAILABLE:
                 repos[self.mInspectionFilter] = self.mRepositories[self.mInspectionFilter]
             return repos
 
-        for i in self.mRepositories:
-            if self.mRepositories[i]["enabled"] and self.mRepositories[i]["valid"] and self.mRepositories[i]["state"] == 3:
-                repos[i] = self.mRepositories[i]
-        return repos
+        return {k: v for k, v in self.mRepositories.items() if v['enabled'] and v['valid'] and v['state'] == Repositories.STATE_UNAVAILABLE}
 
-    # ----------------------------------------- #
-    def urlParams(self):
+    def urlParams(self) -> str:
         """ return GET parameters to be added to every request """
         # Strip down the point release segment from the version string
         return "?qgis={}".format(re.sub(r'\.\d*$', '', pyQgisVersion()))
 
-    # ----------------------------------------- #
-    def setRepositoryData(self, reposName, key, value):
+    def setRepositoryData(self, reposName: str, key: str, value):
         """ write data to the mRepositories dict """
         self.mRepositories[reposName][key] = value
 
-    # ----------------------------------------- #
-    def remove(self, reposName):
+    def remove(self, reposName: str):
         """ remove given item from the mRepositories dict """
         del self.mRepositories[reposName]
 
-    # ----------------------------------------- #
-    def rename(self, oldName, newName):
+    def rename(self, oldName: str, newName: str):
         """ rename repository key """
         if oldName == newName:
             return
         self.mRepositories[newName] = self.mRepositories[oldName]
         del self.mRepositories[oldName]
 
-    # ----------------------------------------- #
-    def checkingOnStart(self):
+    def checkingOnStart(self) -> bool:
         """ return true if checking for news and updates is enabled """
-        settings = QgsSettings()
-        return settings.value(settingsGroup + "/checkOnStart", False, type=bool)
+        return QgsGui.settingsRegistryGui().settingsEntry('plugins/automatically-check-for-updates').value()
 
-    # ----------------------------------------- #
-    def setCheckingOnStart(self, state):
+    def setCheckingOnStart(self, state: bool):
         """ set state of checking for news and updates """
-        settings = QgsSettings()
-        settings.setValue(settingsGroup + "/checkOnStart", state)
+        QgsGui.settingsRegistryGui().settingsEntry('plugins/automatically-check-for-updates').setValue(state)
 
-    # ----------------------------------------- #
-    def checkingOnStartInterval(self):
-        """ return checking for news and updates interval """
-        settings = QgsSettings()
-        try:
-            # QgsSettings may contain non-int value...
-            i = settings.value(settingsGroup + "/checkOnStartInterval", 1, type=int)
-        except:
-            # fallback do 1 day by default
-            i = 1
-        if i < 0:
-            i = 1
-        # allowed values: 0,1,3,7,14,30 days
-        interval = 0
-        for j in [1, 3, 7, 14, 30]:
-            if i >= j:
-                interval = j
-        return interval
-
-    # ----------------------------------------- #
-    def setCheckingOnStartInterval(self, interval):
-        """ set checking for news and updates interval """
-        settings = QgsSettings()
-        settings.setValue(settingsGroup + "/checkOnStartInterval", interval)
-
-    # ----------------------------------------- #
     def saveCheckingOnStartLastDate(self):
         """ set today's date as the day of last checking  """
         settings = QgsSettings()
         settings.setValue(settingsGroup + "/checkOnStartLastDate", QDate.currentDate())
 
-    # ----------------------------------------- #
-    def timeForChecking(self):
+    def timeForChecking(self) -> bool:
         """ determine whether it's the time for checking for news and updates now """
-        if self.checkingOnStartInterval() == 0:
-            return True
         settings = QgsSettings()
         try:
             # QgsSettings may contain ivalid value...
             interval = settings.value(settingsGroup + "/checkOnStartLastDate", type=QDate).daysTo(QDate.currentDate())
         except:
             interval = 0
-        if interval >= self.checkingOnStartInterval():
+        if interval >= Repositories.CHECK_ON_START_INTERVAL:
             return True
         else:
             return False
 
-    # ----------------------------------------- #
     def load(self):
         """ populate the mRepositories dict"""
         self.mRepositories = {}
@@ -316,20 +276,23 @@ class Repositories(QObject):
             self.mRepositories[key]["valid"] = settings.value(key + "/valid", True, type=bool)
             self.mRepositories[key]["Relay"] = Relay(key)
             self.mRepositories[key]["xmlData"] = None
-            self.mRepositories[key]["state"] = 0
+            self.mRepositories[key]["state"] = Repositories.STATE_DISABLED
             self.mRepositories[key]["error"] = ""
         settings.endGroup()
 
-    # ----------------------------------------- #
-    def requestFetching(self, key, url=None, redirectionCounter=0):
+    def requestFetching(self, key: str, url: Optional[QUrl] = None, redirectionCounter=0, force_reload: bool = False):
         """ start fetching the repository given by key """
-        self.mRepositories[key]["state"] = 1
+        self.mRepositories[key]["state"] = Repositories.STATE_LOADING
         if not url:
             url = QUrl(self.mRepositories[key]["url"] + self.urlParams())
         # v=str(Qgis.QGIS_VERSION_INT)
         # url.addQueryItem('qgis', '.'.join([str(int(s)) for s in [v[0], v[1:3]]]) ) # don't include the bugfix version!
 
         self.mRepositories[key]["QRequest"] = QNetworkRequest(url)
+        self.mRepositories[key]["QRequest"].setAttribute(QNetworkRequest.Attribute(QgsNetworkRequestParameters.AttributeInitiatorClass), "Relay")
+        self.mRepositories[key]["QRequest"].setAttribute(QNetworkRequest.FollowRedirectsAttribute, True)
+        if force_reload:
+            self.mRepositories[key]["QRequest"].setAttribute(QNetworkRequest.CacheLoadControlAttribute, QNetworkRequest.AlwaysNetwork)
         authcfg = self.mRepositories[key]["authcfg"]
         if authcfg and isinstance(authcfg, str):
             if not QgsApplication.authManager().updateNetworkRequest(
@@ -346,30 +309,24 @@ class Repositories(QObject):
         self.mRepositories[key]["xmlData"].setProperty('reposName', key)
         self.mRepositories[key]["xmlData"].setProperty('redirectionCounter', redirectionCounter)
         self.mRepositories[key]["xmlData"].downloadProgress.connect(self.mRepositories[key]["Relay"].dataReadProgress)
-        self.mRepositories[key]["xmlData"].finished.connect(self.xmlDownloaded)
+        self.mRepositories[key]["xmlDataFinished"] = self.mRepositories[key]["xmlData"].finished.connect(self.xmlDownloaded)
 
-    # ----------------------------------------- #
-    def fetchingInProgress(self):
-        """ return true if fetching repositories is still in progress """
-        for key in self.mRepositories:
-            if self.mRepositories[key]["state"] == 1:
-                return True
-        return False
+    def fetchingInProgress(self) -> bool:
+        """ return True if fetching repositories is still in progress """
+        return any(v['state'] == Repositories.STATE_LOADING for v in self.mRepositories.values())
 
-    # ----------------------------------------- #
-    def killConnection(self, key):
+    def killConnection(self, key: str):
         """ kill the fetching on demand """
-        if self.mRepositories[key]["state"] == 1 and self.mRepositories[key]["xmlData"] and self.mRepositories[key]["xmlData"].isRunning():
-            self.mRepositories[key]["xmlData"].finished.disconnect()
+        if self.mRepositories[key]["state"] == Repositories.STATE_LOADING and self.mRepositories[key]["xmlData"] and self.mRepositories[key]["xmlData"].isRunning():
+            self.mRepositories[key]["xmlData"].finished.disconnect(self.mRepositories[key]["xmlDataFinished"])
             self.mRepositories[key]["xmlData"].abort()
 
-    # ----------------------------------------- #
     def xmlDownloaded(self):
         """ populate the plugins object with the fetched data """
         reply = self.sender()
         reposName = reply.property('reposName')
-        if reply.error() != QNetworkReply.NoError:                             # fetching failed
-            self.mRepositories[reposName]["state"] = 3
+        if reply.error() != QNetworkReply.NoError:  # fetching failed
+            self.mRepositories[reposName]["state"] = Repositories.STATE_UNAVAILABLE
             self.mRepositories[reposName]["error"] = reply.errorString()
             if reply.error() == QNetworkReply.OperationCanceledError:
                 self.mRepositories[reposName]["error"] += "\n\n" + QCoreApplication.translate("QgsPluginInstaller", "If you haven't canceled the download manually, it was most likely caused by a timeout. In this case consider increasing the connection timeout value in QGIS options window.")
@@ -379,7 +336,7 @@ class Repositories(QObject):
                 redirectionUrl = reply.url().resolved(redirectionUrl)
             redirectionCounter = reply.property('redirectionCounter') + 1
             if redirectionCounter > 4:
-                self.mRepositories[reposName]["state"] = 3
+                self.mRepositories[reposName]["state"] = Repositories.STATE_UNAVAILABLE
                 self.mRepositories[reposName]["error"] = QCoreApplication.translate("QgsPluginInstaller", "Too many redirections")
             else:
                 # Fire a new request and exit immediately in order to quietly destroy the old one
@@ -414,23 +371,32 @@ class Repositories(QObject):
                         trusted = True
                     icon = pluginNodes.item(i).firstChildElement("icon").text().strip()
                     if icon and not icon.startswith("http"):
-                        icon = "http://{}/{}".format(QUrl(self.mRepositories[reposName]["url"]).host(), icon)
+                        url = QUrl(self.mRepositories[reposName]["url"])
+                        if url.scheme() in ('http', 'https'):
+                            icon = "{}://{}/{}".format(url.scheme(), url.host(), icon)
 
                     if pluginNodes.item(i).toElement().hasAttribute("plugin_id"):
                         plugin_id = pluginNodes.item(i).toElement().attribute("plugin_id")
                     else:
                         plugin_id = None
 
+                    version = pluginNodes.item(i).toElement().attribute("version")
+                    download_url = pluginNodes.item(i).firstChildElement("download_url").text().strip()
+
                     plugin = {
                         "id": name,
                         "plugin_id": plugin_id,
                         "name": pluginNodes.item(i).toElement().attribute("name"),
-                        "version_available": pluginNodes.item(i).toElement().attribute("version"),
+                        "version_available": version,
+                        "version_available_stable": version if not experimental else "",
+                        "version_available_experimental": version if experimental else "",
                         "description": pluginNodes.item(i).firstChildElement("description").text().strip(),
                         "about": pluginNodes.item(i).firstChildElement("about").text().strip(),
                         "author_name": pluginNodes.item(i).firstChildElement("author_name").text().strip(),
                         "homepage": pluginNodes.item(i).firstChildElement("homepage").text().strip(),
-                        "download_url": pluginNodes.item(i).firstChildElement("download_url").text().strip(),
+                        "download_url": download_url,
+                        "download_url_stable": download_url if not experimental else "",
+                        "download_url_experimental": download_url if experimental else "",
                         "category": pluginNodes.item(i).firstChildElement("category").text().strip(),
                         "tags": pluginNodes.item(i).firstChildElement("tags").text().strip(),
                         "changelog": pluginNodes.item(i).firstChildElement("changelog").text().strip(),
@@ -440,6 +406,12 @@ class Repositories(QObject):
                         "downloads": pluginNodes.item(i).firstChildElement("downloads").text().strip(),
                         "average_vote": pluginNodes.item(i).firstChildElement("average_vote").text().strip(),
                         "rating_votes": pluginNodes.item(i).firstChildElement("rating_votes").text().strip(),
+                        "create_date": pluginNodes.item(i).firstChildElement("create_date").text().strip(),
+                        "update_date": pluginNodes.item(i).firstChildElement("update_date").text().strip(),
+                        "create_date_stable": pluginNodes.item(i).firstChildElement("create_date").text().strip() if not experimental else "",
+                        "update_date_stable": pluginNodes.item(i).firstChildElement("update_date").text().strip() if not experimental else "",
+                        "create_date_experimental": pluginNodes.item(i).firstChildElement("create_date").text().strip() if experimental else "",
+                        "update_date_experimental": pluginNodes.item(i).firstChildElement("update_date").text().strip() if experimental else "",
                         "icon": icon,
                         "experimental": experimental,
                         "deprecated": deprecated,
@@ -448,12 +420,14 @@ class Repositories(QObject):
                         "installed": False,
                         "available": True,
                         "status": "not installed",
+                        "status_exp": "not installed",
                         "error": "",
                         "error_details": "",
                         "version_installed": "",
                         "zip_repository": reposName,
                         "library": "",
-                        "readonly": False
+                        "readonly": False,
+                        "plugin_dependencies": pluginNodes.item(i).firstChildElement("plugin_dependencies").text().strip(),
                     }
                     qgisMinimumVersion = pluginNodes.item(i).firstChildElement("qgis_minimum_version").text().strip()
                     if not qgisMinimumVersion:
@@ -466,10 +440,10 @@ class Repositories(QObject):
                         if isCompatible(pyQgisVersion(), qgisMinimumVersion, qgisMaximumVersion):
                             # add the plugin to the cache
                             plugins.addFromRepository(plugin)
-                self.mRepositories[reposName]["state"] = 2
+                self.mRepositories[reposName]["state"] = Repositories.STATE_LOADED
             else:
                 # no plugin metadata found
-                self.mRepositories[reposName]["state"] = 3
+                self.mRepositories[reposName]["state"] = Repositories.STATE_UNAVAILABLE
                 if reply.attribute(QNetworkRequest.HttpStatusCodeAttribute) == 200:
                     self.mRepositories[reposName]["error"] = QCoreApplication.translate("QgsPluginInstaller", "Server response is 200 OK, but doesn't contain plugin metatada. This is most likely caused by a proxy or a wrong repository URL. You can configure proxy settings in QGIS options.")
                 else:
@@ -486,18 +460,14 @@ class Repositories(QObject):
 
         reply.deleteLater()
 
-    # ----------------------------------------- #
-    def inspectionFilter(self):
+    def inspectionFilter(self) -> Optional[str]:
         """ return inspection filter (only one repository to be fetched) """
         return self.mInspectionFilter
 
-    # ----------------------------------------- #
-    def setInspectionFilter(self, key=None):
+    def setInspectionFilter(self, key: Optional[str] = None):
         """ temporarily disable all repositories but this for inspection """
         self.mInspectionFilter = key
 
-# --- /class Repositories ---------------------------------------------------------------- #
-
 
 # --- class Plugins ---------------------------------------------------------------------- #
 class Plugins(QObject):
@@ -555,7 +525,7 @@ class Plugins(QObject):
             del self.localCache[key]
 
     # ----------------------------------------- #
-    def removeRepository(self, repo):
+    def removeRepository(self, repo: str):
         """ remove whole repository from the repoCache """
         if repo in self.repoCache:
             del self.repoCache[repo]
@@ -565,7 +535,7 @@ class Plugins(QObject):
         """ get the metadata of an installed plugin """
         def metadataParser(fct):
             """ plugin metadata parser reimplemented from qgis.utils
-                for better control on wchich module is examined
+                for better control of which module is examined
                 in case there is an installed plugin masking a core one """
             global errorDetails
             cp = configparser.ConfigParser()
@@ -581,7 +551,11 @@ class Plugins(QObject):
         def pluginMetadata(fct):
             """ calls metadataParser for current l10n.
                 If failed, fallbacks to the standard metadata """
-            locale = QLocale.system().name()
+            overrideLocale = QgsSettings().value('locale/overrideFlag', False, bool)
+            if not overrideLocale:
+                locale = QLocale.system().name()
+            else:
+                locale = QgsSettings().value('locale/userLocale', '')
             if locale and fct in translatableAttributes:
                 value = metadataParser("{}[{}]".format(fct, locale))
                 if value:
@@ -600,6 +574,10 @@ class Plugins(QObject):
         errorDetails = ""
         version = None
 
+        if not os.path.exists(os.path.join(path, '__init__.py')):
+            error = "broken"
+            errorDetails = QCoreApplication.translate("QgsPluginInstaller", "Missing __init__.py")
+
         metadataFile = os.path.join(path, 'metadata.txt')
         if os.path.exists(metadataFile):
             version = normalizeVersion(pluginMetadata("version"))
@@ -621,7 +599,7 @@ class Plugins(QObject):
         else:
             error = "broken"
             e = errorDetails
-            errorDetails = QCoreApplication.translate("QgsPluginInstaller", u"Error reading metadata")
+            errorDetails = QCoreApplication.translate("QgsPluginInstaller", "Error reading metadata")
             if e:
                 errorDetails += ": " + e
 
@@ -666,18 +644,31 @@ class Plugins(QObject):
             "deprecated": pluginMetadata("deprecated").strip().upper() in ["TRUE", "YES"],
             "trusted": False,
             "version_available": "",
+            "version_available_stable": "",
+            "version_available_experimental": "",
             "zip_repository": "",
             "download_url": path,      # warning: local path as url!
+            "download_url_stable": "",
+            "download_url_experimental": "",
             "filename": "",
             "downloads": "",
             "average_vote": "",
             "rating_votes": "",
+            "create_date": pluginMetadata("create_date"),
+            "update_date": pluginMetadata("update_date"),
+            "create_date_stable": pluginMetadata("create_date_stable"),
+            "update_date_stable": pluginMetadata("update_date_stable"),
+            "create_date_experimental": pluginMetadata("create_date_experimental"),
+            "update_date_experimental": pluginMetadata("update_date_experimental"),
             "available": False,     # Will be overwritten, if any available version found.
             "installed": True,
             "status": "orphan",  # Will be overwritten, if any available version found.
+            "status_exp": "orphan",  # Will be overwritten, if any available version found.
             "error": error,
             "error_details": errorDetails,
-            "readonly": readOnly}
+            "readonly": readOnly,
+            "plugin_dependencies": pluginMetadata("plugin_dependencies"),
+        }
         return plugin
 
     # ----------------------------------------- #
@@ -732,7 +723,12 @@ class Plugins(QObject):
                 # check if the plugin is allowed and if there isn't any better one added already.
                 if (allowExperimental or not plugin["experimental"]) \
                    and (allowDeprecated or not plugin["deprecated"]) \
-                   and not (key in self.mPlugins and self.mPlugins[key]["version_available"] and compareVersions(self.mPlugins[key]["version_available"], plugin["version_available"]) < 2):
+                   and not (
+                       key in self.mPlugins and self.mPlugins[key]["version_available"]
+                       and compareVersions(self.mPlugins[key]["version_available"], plugin["version_available"]) < 2
+                       and self.mPlugins[key]["experimental"] and not plugin["experimental"]
+                ):
+
                     # The mPlugins dict contains now locally installed plugins.
                     # Now, add the available one if not present yet or update it if present already.
                     if key not in self.mPlugins:
@@ -749,36 +745,67 @@ class Plugins(QObject):
                         # other remote metadata is preferred:
                         for attrib in ["name", "plugin_id", "description", "about", "category", "tags", "changelog", "author_name", "author_email", "homepage",
                                        "tracker", "code_repository", "experimental", "deprecated", "version_available", "zip_repository",
-                                       "download_url", "filename", "downloads", "average_vote", "rating_votes", "trusted"]:
+                                       "download_url", "filename", "downloads", "average_vote", "rating_votes", "trusted", "plugin_dependencies",
+                                       "version_available_stable", "version_available_experimental", "download_url_stable", "download_url_experimental",
+                                       "create_date", "update_date", "create_date_stable", "update_date_stable", "create_date_experimental", "update_date_experimental"]:
                             if attrib not in translatableAttributes or attrib == "name":  # include name!
-                                if plugin[attrib]:
+                                if plugin.get(attrib, False):
                                     self.mPlugins[key][attrib] = plugin[attrib]
+
+                    # If the stable version is higher than the experimental version, we ignore the experimental version
+                    if compareVersions(self.mPlugins[key]["version_available_stable"], self.mPlugins[key]["version_available_experimental"]) == 1:
+                        self.mPlugins[key]["version_available_experimental"] = ''
+
                     # set status
                     #
                     # installed   available   status
                     # ---------------------------------------
                     # none        any         "not installed" (will be later checked if is "new")
-                    # any         none        "orphan"
+                    # no          none        "none available"
+                    # yes         none        "orphan"
                     # same        same        "installed"
                     # less        greater     "upgradeable"
                     # greater     less        "newer"
-                    if not self.mPlugins[key]["version_available"]:
+                    if not self.mPlugins[key]["version_available_stable"] and not self.mPlugins[key]["version_installed"]:
+                        self.mPlugins[key]["status"] = "none available"
+                    elif not self.mPlugins[key]["version_available_stable"] and self.mPlugins[key]["version_installed"]:
                         self.mPlugins[key]["status"] = "orphan"
                     elif not self.mPlugins[key]["version_installed"]:
                         self.mPlugins[key]["status"] = "not installed"
                     elif self.mPlugins[key]["version_installed"] in ["?", "-1"]:
                         self.mPlugins[key]["status"] = "installed"
-                    elif compareVersions(self.mPlugins[key]["version_available"], self.mPlugins[key]["version_installed"]) == 0:
+                    elif compareVersions(self.mPlugins[key]["version_available_stable"], self.mPlugins[key]["version_installed"]) == 0:
                         self.mPlugins[key]["status"] = "installed"
-                    elif compareVersions(self.mPlugins[key]["version_available"], self.mPlugins[key]["version_installed"]) == 1:
+                    elif compareVersions(self.mPlugins[key]["version_available_stable"], self.mPlugins[key]["version_installed"]) == 1:
                         self.mPlugins[key]["status"] = "upgradeable"
                     else:
                         self.mPlugins[key]["status"] = "newer"
                     # debug: test if the status match the "installed" tag:
-                    if self.mPlugins[key]["status"] in ["not installed"] and self.mPlugins[key]["installed"]:
-                        raise Exception("Error: plugin status is ambiguous (1)")
+                    if self.mPlugins[key]["status"] in ["not installed", "none available"] and self.mPlugins[key]["installed"]:
+                        raise Exception("Error: plugin status is ambiguous (1) for plugin {}".format(key))
                     if self.mPlugins[key]["status"] in ["installed", "orphan", "upgradeable", "newer"] and not self.mPlugins[key]["installed"]:
-                        raise Exception("Error: plugin status is ambiguous (2)")
+                        raise Exception("Error: plugin status is ambiguous (2) for plugin {}".format(key))
+
+                    if not self.mPlugins[key]["version_available_experimental"] and not self.mPlugins[key]["version_installed"]:
+                        self.mPlugins[key]["status_exp"] = "none available"
+                    elif not self.mPlugins[key]["version_available_experimental"] and self.mPlugins[key]["version_installed"]:
+                        self.mPlugins[key]["status_exp"] = "orphan"
+                    elif not self.mPlugins[key]["version_installed"]:
+                        self.mPlugins[key]["status_exp"] = "not installed"
+                    elif self.mPlugins[key]["version_installed"] in ["?", "-1"]:
+                        self.mPlugins[key]["status_exp"] = "installed"
+                    elif compareVersions(self.mPlugins[key]["version_available_experimental"], self.mPlugins[key]["version_installed"]) == 0:
+                        self.mPlugins[key]["status_exp"] = "installed"
+                    elif compareVersions(self.mPlugins[key]["version_available_experimental"], self.mPlugins[key]["version_installed"]) == 1:
+                        self.mPlugins[key]["status_exp"] = "upgradeable"
+                    else:
+                        self.mPlugins[key]["status_exp"] = "newer"
+                    # debug: test if the status_exp match the "installed" tag:
+                    if self.mPlugins[key]["status_exp"] in ["not installed", "none available"] and self.mPlugins[key]["installed"]:
+                        raise Exception("Error: plugin status_exp is ambiguous (1) for plugin {}".format(key))
+                    if self.mPlugins[key]["status_exp"] in ["installed", "orphan", "upgradeable", "newer"] and not self.mPlugins[key]["installed"]:
+                        raise Exception("Error: plugin status_exp is ambiguous (2) for plugin {} (status_exp={})".format(key, self.mPlugins[key]["status_exp"]))
+
         self.markNews()
 
     # ----------------------------------------- #
